package FluxY_Lib::MS::DataClass;

################################################################################
# This module provides support to the specific compound data classes. Each 
# specific data class provides the details of a given compound and over-rides,
# as necessary, the provided methods.
################################################################################

use strict;
use warnings;

use FluxY_Lib::Chemistry::atomInfo; # User Lib - Atomic Masses.

use base 'Exporter';

my $dataType  = "";   # The name of the set of compounds in the specific class.
my @atomList  = ();   # List of the atoms considerred for natural abundance.
my %compounds = ();   # Hash of compound data.
   # Each compound datum is an array which must contain at least two elements,
   # and in the following order:
   # 1. The compound names, in lower case for standard comparison.
   # 2. The atom count, in the order specified by atomList.


################################################################################
# Class constructor
#
# Used to instantiate an object for this class.
#
################################################################################
sub new {
   my ($class, $cRef, $aRef, $type) = @_;
    
   %compounds = %{$cRef};
   @atomList  = @{$aRef};
	$dataType  = $type;

   my $self = bless {}, $class;
    
   return $self;
}


################################################################################
# Subroutine: GetCompounds
#
# Returns the hash of compound names to key values.
#
################################################################################
sub GetCompounds {
   return %compounds;
}


################################################################################
# Subroutine: GetCompoundList
#
# Returns the array of hash keys for the compound list.
#
################################################################################
sub GetCompoundList {
   return (sort keys %compounds);
}


################################################################################
# Subroutine: GetAtomList
#
# Returns the atomList array.
#
################################################################################
sub GetAtomList {
   return @atomList;
}


################################################################################
# Subroutine: GetDataType
#
# Returns the dataType string.
#
################################################################################
sub GetDataType {
   return $dataType;
}


################################################################################
# Subroutine: GetNumCarbons
#
# Returns the number of carbon atoms for the specified (unfragmented) compound.
#
################################################################################
sub GetNumCarbons {
   my ($self, $compound) = @_;

   # Find the index corresponding to carbon in the atom list:
   my $cIndex = $self->GetAtomPosition("C");
   
   my $numC = @{${$compounds{$compound}}[1]}[$cIndex];
   
   return $numC;
}


################################################################################
# Subroutine: GetCarbonInfo
#
# Returns the number of carbon atoms, and the default positional carbon string
# for the given compound.
# - the default carbon string is complete: 1 .. numC.
#
################################################################################
sub GetCarbonInfo {
   my ($self, $compound) = @_;

   # Check that the key is valid:
   if ( !(exists  $compounds{$compound}) ) { return; }

   my $numCarbons = $self->GetNumCarbons($compound);  
   
   my $atomStr = "1";
   for (my $i=2; $i<=$numCarbons; $i++) { $atomStr .= ",$i"; }
   
   return($numCarbons, $atomStr);
}


################################################################################
# Subroutine: GetAtomCount
#
# Returns the atom count corresponding to the given compound.
#
################################################################################
sub GetAtomCount {
   my ($self, $compound) = @_;

   return @{${$compounds{$compound}}[1]};
}


################################################################################
# Subroutine: PrintMassTables
#
# This is a placeholder method for a required class to print a table containing
# the atom count / fragment mass for each compound.
#
################################################################################
sub PrintMassTables {}


################################################################################
# Subroutine: GetAtomPosition
#
# Returns the position of the specified atom in the atomList array.
# - if not found returns the flag: -1.
#
################################################################################
sub GetAtomPosition {
   my ($self, $atom) = @_;

   # Find the index corresponding to the specified atom in the atom list:
   for (my $i=0; $i < @atomList; $i++) {
      if ($atomList[$i] eq $atom) { return $i }
   }
   
   return -1;
}


1;