package FluxY_Lib::MS::DataClass_SM;

################################################################################
# This module provides support for Soluble Metabolites. Each fragment is listed
# uniquely in the compounds list, with derivatizing information directly
# integrated.
#
# This class inherits basic functionality from the DataClass parent class,
# initializing it with the soluble metabolite compounds, the atom list and
# compound name. It over-rides the GetCarbonInfo() and implements the required
# PrintMassTables(). Two additional helper classes, GenerateMassList() and
# GetMassList() are also implemented.
#
################################################################################

use strict;
use warnings;

use FluxY_Lib::Chemistry::atomInfo; # User Lib - Atomic Masses.
use FluxY_Lib::Math::Math_Utils;    # User Lib - Simple math functions.

use FluxY_Lib::MS::DataClass;       # User Lib - The common parent class.

our @ISA = qw(FluxY_Lib::MS::DataClass);    # Inherits from DataClass


# List of the atoms found in the starch monomer and subsequently considerred for
# natural abundance. 
my @atomList = ('C','H','O','N','S','Si','P');
my @atomMass = ();

# Each of the following atom counts is an array of the number of each corresponding
# atom relative to the order specified in the atomList, i.e. C, H and 0.

# Hash of compound data, (first two array elements are required).
# 1. The compound names (Soluable Metabolites), in lower case (for standard comparison).
# 2. The atom count for the soluable metabolite.
# 3. The starting carbon identifier and ending carbon identifier.
my %compounds = (
	ala3_b1 => ["alanine_beta-_(3TMS)", [7,20,0,1,0,2,0], [3,3]],
	ala3_b2 => ["alanine_beta-_(3TMS)", [11,28,2,1,0,3,0], [1,3]],
	ala2    => ["alanine_(2TMS)", [5,14,0,1,0,1,0], [2,3]],
	ala3_b  => ["alanine_(3TMS)", [11,28,2,1,0,3,0], [1,3]],
	asp3_1  => ["aspartic_acid_(3TMS)", [8,20,2,1,0,2,0], [1,2]],
	asp3_2  => ["aspartic_acid_(3TMS)", [9,22,2,1,0,2,0], [2,4]],
	but_1   => ["butyric_acid,_4-amino-_(3TMS)", [7,20,0,1,0,2,0], [4,4]],
	but_2   => ["butyric_acid,_4-amino-_(3TMS)", [12,30,2,1,0,3,0], [1,4]],
	glu_1   => ["glutamic_acid_(3TMS)", [4,6,1,1,0,0,0], [2,5]],
	glu_1   => ["glutamic_acid_(3TMS)", [10,24,2,1,0,2,0], [2,5]],
	glu_1   => ["glutamic_acid_(3TMS)", [13,30,4,1,0,3,0], [1,5]],
	gly2_1  => ["glycine_(2TMS)", [4,12,0,1,0,1,0], [2,2]],
	gly2_2  => ["glycine_(2TMS)", [7,18,2,1,0,2,0], [1,2]],
	gly3_1  => ["glycine_(3TMS)", [7,20,0,1,0,2,0], [2,2]],
	gly3_2  => ["glycine_(3TMS)", [10,26,2,1,0,3,0], [1,2]],
	his3_1  => ["histidine_(3TMS)", [8,20,2,1,0,2,0], [1,2]],
	his3_2  => ["histidine_(3TMS)", [11,24,0,3,0,2,0], [2,6]],
	ile2_1  => ["isoleucine_(2TMS)", [8,20,0,1,0,1,0], [2,6]],
	ile2_2  => ["isoleucine_(2TMS)", [8,20,2,1,0,2,0], [1,2]],
	leu2_1  => ["leucine_(2TMS)", [8,20,0,1,0,1,0], [2,6]],
	leu2_2  => ["leucine_(2TMS)", [8,20,2,1,0,2,0], [1,2]],
	lys4_1  => ["lysine_(4TMS)", [7,20,0,1,0,2,0], [6,6]],
	lys4_2  => ["lysine_(4TMS)", [14,37,0,2,0,3,0], [2,6]],
	met2    => ["methionine_(2TMS)", [7,18,0,1,1,1,0], [2,5]],
	orn3_1  => ["ornithine_(3TMS)", [4,12,0,1,0,1,0], [5,5]],
	orn3_2  => ["ornithine_(3TMS)", [14,36,2,2,0,3,0], [1,5]],
	orn4_1  => ["ornithine_(4TMS)", [7,20,0,1,0,2,0], [5,5]],
	orn4_2  => ["ornithine_(4TMS)", [13,35,0,2,0,3,0], [2,5]],
	phe2_1  => ["phenylalanine_(2TMS)", [7,7,0,0,0,0,0], [3,9]],
	phe2_2  => ["phenylalanine_(2TMS)", [11,18,0,1,0,1,0], [2,9]],
	phe2_3  => ["phenylalanine_(2TMS)", [8,20,2,1,0,2,0], [1,2]],
	pro2    => ["proline_(2TMS)", [7,16,0,1,0,1,0], [2,5]],
	ser3_1  => ["serine_(3TMS)", [8,22,1,1,0,2,0], [2,3]],
	ser3_2  => ["serine_(3TMS)", [8,20,2,1,0,2,0], [1,2]],
	ser3_3  => ["serine_(3TMS)", [11,28,3,1,0,3,0], [1,3]],
	thr3_1  => ["threonine_(3TMS)", [5,13,1,0,0,1,0], [3,4]],
	thr3_2  => ["threonine_(3TMS)", [12,30,3,1,0,3,0], [1,4]],
	try3_1  => ["tryptophan_(3TMS)", [12,16,0,1,0,1,0], [3,11]],
	try3_2  => ["tryptophan_(3TMS)", [8,20,2,1,0,2,0], [1,2]],
	tyr3_1  => ["tyrosine_(3TMS)", [10,15,1,0,0,1,0], [3,9]],
	tyr3_2  => ["tyrosine_(3TMS)", [8,20,2,1,0,2,0], [1,2]],
	tyr3_3  => ["tyrosine_(3TMS)", [14,26,1,1,0,2,0], [2,9]],
	val2_1  => ["valine_(2TMS)", [7,18,0,1,0,1,0], [2,5]],
	val2_2  => ["valine_(2TMS)", [8,20,2,1,0,2,0], [1,2]],
	ca4_1   => ["citric_acid_(4TMS)", [14,27,6,0,0,3,0], [1,6]],
	ca4_2   => ["citric_acid_(4TMS)", [17,37,7,0,0,4,0], [1,6]],
	ica4_1  => ["iso-citric_acid_(4TMS)", [14,27,6,0,0,3,0], [1,6]],
	ica4_2  => ["iso-citric_acid_(4TMS)", [17,37,7,0,0,4,0], [1,6]],
	fa2     => ["fumaric_acid_(2TMS)", [9,17,4,0,0,2,0], [1,4]],
	ga2     => ["glutaric_acid,_2-oxo-_(2TMS)", [11,22,5,1,0,2,0], [1,5]],
	gla3    => ["glyceric_acid_(3TMS)", [11,27,4,0,0,3,0], [1,3]],
	gla2    => ["glycolic_acid_(2TMS)", [7,17,3,0,0,2,0], [1,2]],
	la2     => ["lactic_acid_(2TMS)", [8,19,3,0,0,2,0], [1,3]],
	ma3_1   => ["malic_acid_(3TMS)", [9,17,4,0,0,2,0], [1,4]],
	ma3_2   => ["malic_acid_(3TMS)", [12,27,5,0,0,3,0], [1,4]],
	sa2_1   => ["succinic_acid_(2TMS)", [7,12,3,0,0,1,0], [1,4]],
	sa2_2   => ["succinic_acid_(2TMS)", [9,19,4,0,0,2,0], [1,4]],
	fru5_1  => ["fructose_(5TMS)", [4,11,1,0,0,1,0], [6,6]],
	fru5_2  => ["fructose_(5TMS)", [8,21,2,0,0,2,0], [5,6]],
	fru5_3  => ["fructose_(5TMS)", [9,21,2,0,0,2,0], [4,6]],
	fru5_4  => ["fructose_(5TMS)", [10,24,3,1,0,2,0], [1,3]],
	fru5_5  => ["fructose_(5TMS)", [12,31,3,0,0,3,0], [4,6]],
	fru5_6  => ["fructose_(5TMS)", [14,34,4,1,0,3,0], [1,4]],
	glyc3_1 => ["glycerol_(3TMS)", [12,32,3,0,0,3,0], [1,3]],
	glyc3_2 => ["glycerol_(3TMS)", [11,29,3,0,0,3,0], [1,3]],
	glyc3_3 => ["glycerol_(3TMS)", [9,22,2,0,0,2,0], [1,3]],
	glc5_1  => ["glucose_(5TMS)", [4,11,1,0,0,1,0], [6,6]],
	glc5_2  => ["glucose_(5TMS)", [6,14,2,1,0,1,0], [1,2]],
	glc5_3  => ["glucose_(5TMS)", [8,21,2,0,0,2,0], [5,6]],
	glc5_4  => ["glucose_(5TMS)", [9,21,2,0,0,2,0], [4,6]],
	glc5_5  => ["glucose_(5TMS)", [10,24,3,1,0,2,0], [1,3]],
	glc5_6  => ["glucose_(5TMS)", [13,31,3,0,0,3,0], [3,6]],
	glc5_7  => ["glucose_(5TMS)", [14,34,4,1,0,3,0], [1,4]],
	suc8_1  => ["sucrose_(8TMS)", [4,11,1,0,0,1,0], [6,6]],
	suc8_2  => ["sucrose_(8TMS)", [9,21,2,0,0,2,0], [4,6]],
	suc8_3  => ["sucrose_(8TMS)", [13,31,3,0,0,3,0], [3,6]],
	suc8_4  => ["sucrose_(8TMS)", [15,33,4,0,0,3,0], [1,6]],
	suc8_5  => ["sucrose_(8TMS)", [18,43,5,0,0,4,0], [1,6]],
	sorb_1  => ["sorbitol_(6TMS)", [4,11,1,0,0,1,0], [6,6]],
	sorb_3  => ["sorbitol_(6TMS)", [8,21,2,0,0,2,0], [5,6]],
	sorb_4  => ["sorbitol_(6TMS)", [9,21,2,0,0,2,0], [4,6]],
	sorb_6  => ["sorbitol_(6TMS)", [13,31,3,0,0,3,0], [3,6]],
	fru6p6  => ["fructose-6-phosphate_(6TMS)", [9,21,2,0,0,2,0], [4,6]],
	glc6p6_1 => ["glucose-6-phosphate_(6TMS)", [4,11,1,0,0,1,0], [6,6]],
	glc6p6_2 => ["glucose-6-phosphate_(6TMS)", [6,14,2,1,0,1,0], [1,2]],
	glc6p6_3 => ["glucose-6-phosphate_(6TMS)", [9,21,2,0,0,2,0], [4,6]],	
	hex6p6_1 => ["hexose-phosphate_(6TMS)", [4,11,1,0,0,1,0], [6,6]],
	hex6p6_2 => ["hexose-phosphate_(6TMS)", [6,14,2,1,0,1,0], [1,2]],
	hex6p6_3 => ["hexose-phosphate_(6TMS)", [9,21,2,0,0,2,0], [4,6]],
	pyr3	  => ["pyruvic_acid_(3tms)", [6,12,3,1,0,1,0], [1,3]],
	ga3p4_1 => ["glyceric_acid-3-phosphate_(4TMS)", [9,21,2,0,0,2,0], [1,3]],
	ga3p4_2 => ["glyceric_acid-3-phosphate_(4TMS)", [14,36,7,0,0,4,1], [1,3]],
	pep3_1  => ["phosphoenolpyruvate_(3TMS)", [9,21,2,0,0,2,0], [1,3]],
	pep3_2  => ["phosphoenolpyruvate_(3TMS)", [11,26,6,0,0,3,1], [1,3]],
   );
   
# A 2D Hash of compound masses, indexed by compunds key and frags keys.
# Each element is the truncated mass value for the given compound/fragment.
my %massList = ();


################################################################################
# Class constructor
#
# Used to instantiate an object for this class.
#
################################################################################
sub new {
   my ($class) = @_;

   # Initialize the atom mass vector.
   @atomMass = &GetElementMasses(@atomList);

   # Initialize the compound mass list.
   GenerateMassList();

   # Call the constructor of the parent class, Person.
   my $self = $class->SUPER::new(\%compounds, \@atomList, "Soluble Metabolites (SM)");
   
   bless $self, $class;

   return $self;
}


########################################################
#                                                      #
#   Over-ridden DataClass methods.                     #
#                                                      #
#   - to support the derivating agent and fragments.   #
#                                                      #
########################################################

################################################################################
# Subroutine: GetCarbonInfo
#
# Returns the number of carbon atoms, and the default positional carbon string
# for the given compound.
#
################################################################################
sub GetCarbonInfo {
   my ($self, $compound) = @_;

   # Check that the key is valid:
   if ( !(exists  $compounds{$compound}) ) { return; }
   
   my ($firstC,$lastC) = @{${$compounds{$compound}}[2]};
	
	my $numCarbons = $lastC - $firstC + 1;

   my $atomStr = $firstC;
   for (my $i=$firstC+1; $i<=$lastC; $i++) { $atomStr .= ",$i"; }
   
   return($numCarbons, $atomStr);
}


######################################
#                                    #
#   Implemented DataClass methods.   #
#                                    #
######################################

################################################################################
# Subroutine: PrintMassTables
#
# Prints a table containing the atom count / mass for each compound.
# - Added carbon backbone & full name.
#
################################################################################
sub PrintMassTables{
   my $outFile = "SM_Data_Table.txt";

   open (FILE, ">" . $outFile) or FileSystemError(900, $outFile, $!);

   print FILE "Soluble Metabolite Data Table:\n\n\t";

   print FILE "\t" . join("\t", @atomList) . "\tMass\tC-Backbone\tFull Name\n";

   foreach my $compound (sort keys %compounds) {   
      print FILE $compound . "\t";
      
      my $fullName        = ${$compounds{$compound}}[0];
      my @totAtoms        = @{${$compounds{$compound}}[1]};
		my ($firstC,$lastC) = @{${$compounds{$compound}}[2]};
      
      print FILE "\t" .  join("\t", @totAtoms) . "\t" . $massList{$compound}{'M-0'} . "\t" . $firstC . "-" . $lastC . "\t" . $fullName . "\n";
   }
   
   close(FILE)                 or FileSystemError(901, $outFile, $!);
}


#######################
#                     #
#   Public methods.   #
#                     #
#######################

################################################################################
# Subroutine: GenerateMassList
#
# Generates a list of mass values in a hash (indexed by compound key) of hashes
# (indexed by fragment key).
#
################################################################################
sub GenerateMassList {
   foreach my $compound (sort keys %compounds) {
   
      my %list = ();
      
      my @totAtoms   = @{${$compounds{$compound}}[1]};
      
      my $mass = 0;
      for(my $i=0; $i<@totAtoms; $i++) {
         $mass += ($totAtoms[$i] * $atomMass[$i]);
      }
      
      $list{'M-0'} = &Trunc($mass);
      
      $massList{$compound} = \%list;
   }
}


################################################################################
# Subroutine: GetMassList
#
# Return the massList hash.
#
################################################################################
sub GetMassList {
   return %massList;
}

1;
